//  Drivetrain.h - manages the engine, clutch, transmission and differential.
//
//  Copyright (C) 2001 Sam Varner
//
//  This file is part of Vamos Automotive Simulator.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _DRIVETRAIN_H_
#define _DRIVETRAIN_H_

#include <vamos/body/Engine.h>
#include <vamos/body/Clutch.h>
#include <vamos/body/Transmission.h>
#include <vamos/body/Differential.h>

namespace Vamos_Body
{
  //* The drivetrain manages the engine, clutch, transmission and
  //  differential.
  class Drivetrain
  {
	//** The components that make up the drivetrain.

	Engine m_engine;

	Clutch m_clutch;

	Transmission m_transmission;

	Differential m_differential;


	// The throttle position.
	double m_gas;

  public:
	//** Constructor
	Drivetrain (const Engine& engine, 
				const Clutch& clutch, 
				const Transmission& transmission,
				const Differential& differential);

	// Process the input parameters.
	void input (double gas, double clutch,
				double left_wheel_speed, double right_wheel_speed);

	void find_forces ();

	// Advance the drivetrain in time by TIME.
	void propagate (double time);

	void rewind ();

	// Allow access to the components.
	Engine& engine () { return m_engine; }
	Transmission& transmission () { return m_transmission; }

	// Return the torque for the left driven wheel.
	double left_torque () const { return m_differential.left_torque (); }

	// Return the torque for the right driven wheel.
	double right_torque () const { return m_differential.right_torque (); }
  };
}

#endif // !_DRIVETRAIN_H_
