//	Vamos Automotive Simulator
//  Copyright (C) 2001--2002 Sam Varner
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _SURFACE_H_
#define _SURFACE_H_

#include <vamos/geometry/Texture_Image.h>

#include <string>

namespace Vamos_Geometry
{
  class Surface
  {
	double m_right;
	double m_left;
	double m_height;
	bool m_vertical;

	// The frictional properties of the car are muliplied by these factors
	// for this particular surface.  They should all be 1.0 for pavement.
	double m_friction_factor;
	double m_rolling_resistance_factor;
	double m_drag_factor;

	double m_restitution_factor;

	// The texture that gets mapped to this surface.
	Texture_Image* mp_texture_image;

  public:
	// The function of the second arg depends on whether vertical is true
	// or false.
	Surface (double right, double height_or_width, 
			 double friction, double rolling, double drag, double restitution,
			 std::string image_file = "", 
			 bool vertical = false);
	Surface ();
	Surface (const Surface& surf);
	~Surface ();

	const Texture_Image& texture_image () const { return *mp_texture_image; }
	double left () const { return m_left; }
	double right () const { return m_right; }
	double width () const { return m_left - m_right; }
	double height () const { return m_height; }
	bool vertical () const { return m_vertical; }

	double friction_factor () const { return m_friction_factor; }
	double rolling_resistance_factor () const 
	{ return m_rolling_resistance_factor; }
	double drag_factor () const { return m_drag_factor; }
	double restitution_factor () const { return m_restitution_factor; }
  };
}

#endif
