//	Vamos Automotive Simulator
//  Copyright (C) 2001--2002 Sam Varner
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef _TRACK_H_
#define _TRACK_H_

#include <vamos/geometry/Three_Vector.h>
#include <vamos/track/Road_Segment.h>

#include <vector>

namespace Vamos_Track
{
  // An exception thrown when the track can not be adjusted to make a circuit.
  class Can_Not_Close {};

  class Track
  {
	// bounding dimensions in pixels.
	double m_min_x;
	double m_max_x;
	double m_min_y;
	double m_max_y;

	// road surface defaults
	double m_road_width;
	double m_border_width;
	double m_static_friction;
	double m_sliding_friction;
	double m_resolution;

	std::vector <double> m_timing_lines;
	double m_speed_trap_start;
	double m_speed_trap_end;

  protected:
	std::vector <Road_Segment*> m_segments;

  public:
	Track ();
	virtual ~Track ();

	void add_segment (Road_Segment* seg);
	virtual void draw () const {};
	const std::vector <Road_Segment*>& segments () const { return m_segments; }

	// Make the track.
	void build ();
	// This method builds the track and then adjusts the last curve
	// and the straights before and after it to make the track end
	// where it began.
	void build_circuit ();

	void timing_line (double dist) { m_timing_lines.push_back (dist); }
	void speed_trap (double start, double end);
	const std::vector <double>& timing_lines () { return m_timing_lines; }

	double min_x () const { return m_min_x; }
	double max_x () const { return m_max_x; }
	double min_y () const { return m_min_y; }
	double max_y () const { return m_max_y; }
  };
}

#endif
